; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; ndivide

; ---conformance---
; Sketchy Core

; ---purpose---
; Divide two natural numbers, giving a
; result of the form (quotient remainder).

; ---args---
; A - natural number (dividend)
; B - natural number (divisor)

; ---keywords---
; NDIVIDE function, quotient, modulus, division, divide
; arithmetics

; ---see-also---
; digits, divide, divide, nquotient, nremainder, n+, n-, n*

; ---example---
; (ndivide 11 3) => '(3 2)

(define ndivide #t)

:c lib/digits.l
:c lib/caar.l cadr
:c lib/normalize.l
:c lib/nplus.l n+
:c lib/nless.l n<
:c lib/equalp.l equal?
:c lib/nminus.l n-
:c lib/zerop.l zero?
:c lib/reverse.l
:c lib/append.l
:c lib/numbertolist.l number->list
:c lib/listtonumber.l list->number

; ---code---
(define ndivide (lambda (a b)
	(letrec*

		; Equalize the divisor by shifting it to the left
		; (multiplying it by 10) until it has the same number
		; of digits as the dividend.
		; A - list of digits (dividend)
		; B - reverse list of digits (divisor)
		; R - result of the form (new divisor . one I per shift)
		((eql (lambda (a b r)
			(cond
				; end of dividend?
				((null? a)
					; reverse the divisor
					(cons (reverse (car r)) (cdr r)))
				; end of divisor?
				((null? b)
					(eql (cdr a) '()			; go to next digit
						(cons (cons 0-digit		; prepend 0 to divisor
								(car r))
							(cons 'i (cdr r)))))	; count shift
				; default: copy a digit
				(#t (eql (cdr a) (cdr b)
					(cons (cons (car b) (car r)) (cdr r)))))))

		; Divide two lists of digits which have a quotient
		; less than 10.
		; A - dividend
		; B - divisor
		; R - result of the form (A/B*B . A/B)
		(div10 (lambda (a b r)
			(cond ((n< (car r) (list->number a))
					(div10 a b
						(cons (n+ (car r) (list->number b))
							(n+ (cdr r) 1))))
				((equal? (car r) (list->number a)) r)
				(#t (cons (n- (car r) (list->number b))
						(n- (cdr r) 1))))))

		; X / 10
		(d10 (lambda (x)
			(reverse (cdr (reverse x)))))

		; X * 10
		(x10 (lambda (x)
			(append x (list 0-digit))))

		; Divide two lists of digits
		; A - list of digits (dividend)
		; B - divisor of the form (EQL A B ...)
		; R - result (list of digits in normal order)
		(div (lambda (a b r)
			(cond ((null? (cdr b))
					(list (normalize (number->list r)) a))
				(#t (letrec* ((quot
						(div10 a (car b) (cons 0 0))))
					(div (number->list (n- (list->number a) (car quot)))
						(cons (d10 (car b)) (cddr b))
						(n+ (x10 r) (cdr quot)))))))))

		(cond ((zero? b) (bottom 'divide-by-zero))
			((n< a b) (list 0 a))
			(#t (letrec* ((res (div (number->list a)
						(eql (number->list a)
						(number->list b) '(() i)) 0)))
					(list (list->number (car res))
						(list->number (cadr res)))))))))

