; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; normalize

; ---conformance---
; Sketchy Core

; ---purpose---
; Normalize a list of digits by removing leading zeroes
; and plus signs.

; ---args---
; X - list of digits

; ---keywords---
; NORMALIZE function, normalize, normal form, canonical number
; GCD, reducing, numbers, digits

; ---see-also---
; digits, n-, ndivide

; ---example---
; (normalize (list '+ 0-digit 0-digit 0-digit 5-digit 1-digit)) => '(5 1)

(define normalize #t)

:c lib/digits.l
:c lib/caar.l cdar
:c lib/caaar.l caadr cdadr
:c lib/gcd.l
:c lib/equal.l =
:c lib/zerop.l zero?
:c lib/quotient.l
:c lib/minus.l -

; ---code---
(define normalize (lambda (x)
	(letrec*
		; remove leading zeroes from natural number
		((norm (lambda (x)
			(cond ((null? (cdr x)) x)
				((eq? (car x) 0-digit)
					(norm (cdr x)))
				(#t x)))))
		(cond ((eq? (car x) '+)
				(norm (cdr x)))
			((eq? (car x) '-)
				(cons '- (norm (cdr x))))
			(#t (norm x))))))

