; SketchyLISP Library
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---name---
; n*

; ---conformance---
; Sketchy Core

; ---purpose---
; Compute the product of two natural numbers.

; ---args---
; A - natural number
; B - natural number

; ---keywords---
; N* function, times, multiply, multiplication, product
; arithmetics, natural

; ---see-also---
; digits, n+, n-, nquotient, nremainder, *

; ---example---
; (n* 4 5) => 20

(define ntimes #t)

:c lib/digits.l
:c lib/nplus.l n+
:c lib/pred.l
:c lib/append.l
:c lib/reverse.l
:c lib/zerop.l zero?
:c lib/numbertolist.l number->list
:c lib/listtonumber.l list->number

; ---code---
(define n* (lambda (a b)
	(letrec*

		; X*10 where X=/=0
		((x10 (lambda (x)
			(append x (list 0-digit))))

		; Add A to R B times.
		; A,R are numbers, B is a decimal digit.
		(addn (lambda (a b r)
			(cond ((eq? b 0-digit) r)
				(#t (addn a (pred b)
						(n+ (list->number a) r))))))

		; R=A*B
		; B is in reverse order.
		(tms (lambda (a b r)
			(cond ((null? b) r)
				(#t (tms (x10 a) (cdr b) (addn a (car b) r)))))))

		; avoid leading zeroes in result
		(cond ((zero? a) 0)
			(#t (tms (number->list a)
					(reverse (number->list b)) 0))))))

