; SketchyLISP Example Program
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---purpose---
; Print the lyrics of "99 bottles of beer on the wall".

; ---keywords---
; BOTTLES, 99 bottles of beer, example programs

; ---example---
; (bottles)

; ---code---
; Bottles Song in Pure LISP
; Copyright (C) 2003,2005 Nils M Holm <nmh@t3x.org>
;
; This is purely symbolic lisp.
; There are no pre-defined numbers, not even digits.
; There are no strings.
; There is no input/output.
; There are no side effects.
;
; The below expression reduces to a list containing
; sublists. Each sublist contains one verse of the song.
; Each verse consists of four sublists. Each sublist of
; a verse contains one sentence.
; Numbers are represented by lists of digits.
;
; For example, the first verse of the song looks like
; this:
;
; (((9 9) bottles of beer on the wall)
;  ((9 9) bottles of beer)
;  (take one down and pass it around)
;  ((9 8) bottles of beer on the wall))
;
; Required subroutines:
; LETREC*, LAMBDA, QUOTE, CONS, CAR, CDR, EQ?, NULL?, COND
;
; Here we go...

:i off

(define bottles (lambda ()
	(letrec*
	
		; Define some sentences
		((beer-on-wall
			(quote (bottles of beer on the wall)))
		(beer
			(quote (bottles of beer)))
		(take-down
			(quote (take one down and pass it around)))
	
		; Append two lists
		(append (lambda (a b)
			(cond ((null? a) b)
				(#t (cons (car a) (append (cdr a) b))))))
	
		; Append four lists
		(append4 (lambda (a b c d)
			(append a (append b (append c d)))))
	
		; Find the predecessor of a digit,
		; where (pred (quote 0)) => '()
		(pred (lambda (n)
			(cond
				((eq? n (quote 1)) (quote 0))
				((eq? n (quote 2)) (quote 1))
				((eq? n (quote 3)) (quote 2))
				((eq? n (quote 4)) (quote 3))
				((eq? n (quote 5)) (quote 4))
				((eq? n (quote 6)) (quote 5))
				((eq? n (quote 7)) (quote 6))
				((eq? n (quote 8)) (quote 7))
				((eq? n (quote 9)) (quote 8))
				(#t '()))))
	
		; The car of the cdr of a list
		(cadr (lambda (x) (car (cdr x))))
	
		; Form a one-element list
		(olist (lambda (x) (cons x '())))
	
		; Decrement a two-digit natural number
		(decrement (lambda (n)
			(cond ((null? (pred (cadr n)))
					(cons (pred (car n)) (olist (quote 9))))
				(#t
					(cons (car n) (olist (pred (cadr n))))))))
	
		; Check whether n equals (quote (0 0))
		(zerop (lambda (n)
			(cond ((eq? (car n) (quote 0))
					(cond ((eq? (cadr n) (quote 0)) #t)
							(#t #f)))
				(#t #f))))
	
		; Form a verse
		(verse (lambda (n)
			(append4
				(olist (append (olist n) beer-on-wall))
				(olist (append (olist n) beer))
				(olist take-down)
				(olist (append (olist (decrement n)) beer-on-wall)))))
	
		; Collect the verses in a tail-recursive function
		(count-bottles (lambda (n lyrics)
			(cond ((zerop n) lyrics)
				(#t (count-bottles (decrement n)
					(append lyrics (olist (verse n)))))))))
	
		(count-bottles (quote (9 9)) '()))))

