; SketchyLISP Example Program
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---purpose---
; Re-name variables of lambda expressions.
; Lambda variables are re-name in such a way
; that no conflicts can arise between names
; of nested lambda expressions. After
; LVRENAME-ing an expression, full beta
; reduction can be done using SUBST.
; --
; For instance,
; -
; ((LVRENAME '(LAMBDA (X) (LAMBDA (X) X)))
; -
; => (LAMBDA (X:0) (LAMBDA (X:1) X:1))

; ---args---
; X - S-expression

; ---keywords---
; LVRENAME function, lambda abstraction, beta reduction
; lambda variables, lambda variable renaming, example

; ---example---
; (lvrename '(lambda (x) (lambda (x) x))) => '(lambda (x:0) (lambda (x:1) x:1))


:l lib/assoc.l
:l lib/reverse.l
:l lib/append.l
:l lib/neqp.l
:l lib/not.l
:l lib/symbolp.l symbol?
:l lib/listp.l list?
:l lib/plus.l +
:l lib/caar.l cadr cddr
:l lib/caaar.l caddr
:l lib/numbertolist.l number->list

; ---code---
(define lvrename (lambda (x)
	(letrec*

	; Re-name X to first match of NLIST
	((rename (lambda (x nlist)
		(letrec* ((nx (assoc x nlist)))
		(cond ((null? nx) x) (#t (cdr nx))))))

	; Re-name each symbol in L
	(rename-list (lambda (l nlist r)
		(cond ((null? l) (reverse r))
			(#t (rename-list (cdr l) nlist
				(cons (rename (car l) nlist) r))))))

	; Create a binding of the form
	; (NAME . UNIQUE-NAME)
	; Unique names are created by appending -LEVEL
	(make-assoc (lambda (name level)
		 (cons name
			(implode (append (explode name)
						(append '(:) (number->list level)))))))

	; Create a set of local (unique) names
	; OLD   - NLIST so far
	; NEW   - list of symbols to add to NLIST
	; LEVEL - level of nested LAMBDA
	(localnames (lambda (old new level)
		(cond ((null? new) old)
			(#t (cons (make-assoc (car new) level)
				(localnames old (cdr new) level))))))

	; Check if X is a valid lambda expression
	(lambda-p (lambda (x)
		(cond
			((neq? (car x) 'lambda) #f)
			((null? (cdr x)) #f)
			((not (list? (cdr x))) #f)
			((not (list? (cadr x))) #f)
			(#t (not (null? (cddr x)))))))

	; Re-name all lambda variables in X.
	; NLIST holds the bindings used to re-name symbols.
	; LEVEL is the current nesting level of LAMBDA.
	(lvr (lambda (x nlist level)
		(cond
			((null? x) '())
			((symbol? x) (rename x nlist))
			((lambda-p x)
				(letrec* ((new-nlist (localnames nlist (cadr x) level)))
				(list 'lambda
						(rename-list (cadr x) new-nlist '())
						(lvr (caddr x) new-nlist (+ 1 level)))))
			(#t (cons (lvr (car x) nlist level)
					(lvr (cdr x) nlist level)))))))

	(lvr x '() 0))))
