; SketchyLISP Example Program
; Copyright (C) 2005 Nils M Holm. All rights reserved.
; See the file LICENSE of the SketchyLISP distribution
; for conditions of use.

; ---purpose---
; Convert a LABEL into an equivalent LAMBDA expression.
; --
; This program proves that any LABEL can be converted
; into an equivalent LAMBDA expression by automating
; this process.
; --
; Notice that
; --
; ((UNLABEL UNLABEL) UNLABEL)
; --
; breaks SketchyLISP, since it does not perform full
; beta-reduction.

; ---args---
; X - expression to convert

; ---keywords---
; LABEL, LAMBDA, scoping, example

; ---example---
; (unlabel '(letrec* ((a 5) (b 7)) (plus a b))) => '((lambda (a b) (plus a b)) 5 7)

:l lib/reverse.l
:l lib/append.l
:l lib/caar.l cadr
:l lib/caaar.l caddr cadar
:l lib/symbolp.l

; ---code---
(define unlabel (lambda (x)
	(letrec*

	; Create list of symbols to bind in LABEL
	((symlist (lambda (x r)
		(cond ((null? x) (reverse r))
			(#t (symlist (cdr x) (cons (caar x) r))))))

	; Create list of (unlabeled) values to bind
	(vallist (lambda (x r)
		(cond ((null? x) (reverse r))
			(#t (vallist (cdr x) (cons (ulbl (cadar x)) r))))))

	; Extract (unlabeled) term of label
	(term (lambda (x) (ulbl (caddr x))))

	; Create lambda expression
	(makelambda (lambda (x)
		(append (list (list 'lambda (symlist (cadr x) '()) (term x)))
			(vallist (cadr x) '()))))

	; Traverse expression tree, replacing letrec*s with lambda
	(ulbl (lambda (x)
		(cond ((symbol? x) x)
			((null? x) x)
			((eq? (car x) 'quote) x)
			((eq? (car x) 'letrec*) (makelambda x))
			(#t (cons (ulbl (car x)) (ulbl (cdr x))))))))

	(ulbl x))))
